#ifndef IPGAUSSIAN_INC
#define IPGAUSSIAN_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to apply a gaussian smoothing filter

    This class implements a gaussian smoothing filter. The size of the filter window
    can be provided (default 3x3), and the variance is deduced from the kernel size: sigma = size/5. 
    Boundary pixels are processed using mirror interpolation. Used for the Multiscale Retinex.
   
    @author Guillaume Heusch (heusch@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipGaussian : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// size of the gaussian kernel
	int size;
	int halfsize;

	// the pre-computed gaussian kernel
	real *kernel;
	
public:
	/// variance of the kernel
	real sigma; 

	//-----
	
	/** constructor
	    
	    @param size_ is the size of the kernel
	    @param width_ is the width of the input image
	    @param height_ is the height of the input image
	    @param coding_ is the coding format of the input image
	 */
	ipGaussian(int size_, int width_, int height_, const char *coding_);
	
	///
	virtual ~ipGaussian();

	///
	void process(Sequence *seq_in);	
};

}

#endif
