#include "ipNormBestFitLinear.h"

namespace Torch {

ipNormBestFitLinear::ipNormBestFitLinear(int width_, int height_, const char *coding_) : ipCore(coding_)
{
   	width = width_;
        height = height_;

	seq_out = new(allocator) Sequence(2, width_*height_*n_planes);
	
	//
   	mat = new fastMatrix(3);
	
	//
	X = (real *) allocator->alloc(3 * sizeof(real));
	U = (real *) allocator->alloc(3 * sizeof(real));
	W = (real *) allocator->alloc(3 * sizeof(real));
	
	//
	A = (real **) allocator->alloc(3 * sizeof(real *));
	A[0] = (real *) allocator->alloc(3 * sizeof(real));
	A[1] = (real *) allocator->alloc(3 * sizeof(real));
	A[2] = (real *) allocator->alloc(3 * sizeof(real));

	//
	Macc = (real **) allocator->alloc(3 * sizeof(real *));
	Macc[0] = (real *) allocator->alloc(3 * sizeof(real));
	Macc[1] = (real *) allocator->alloc(3 * sizeof(real));
	Macc[2] = (real *) allocator->alloc(3 * sizeof(real));

	//
	A_1 = (real **) allocator->alloc(3 * sizeof(real *));
	A_1[0] = (real *) allocator->alloc(3 * sizeof(real));
	A_1[1] = (real *) allocator->alloc(3 * sizeof(real));
	A_1[2] = (real *) allocator->alloc(3 * sizeof(real));
}

void ipNormBestFitLinear::process(Sequence *seq_in)
{
	int input_size = width * height;
	
	real *src = seq_in->frames[0];
	real *dst = seq_out->frames[0];
	real *linearplane = seq_out->frames[1];
	real mean_src;
	real a, b, c;
	
	switch(coding_id)
	{
	case 0:
		mat->initVector(X);
		mat->initVector(U);
		mat->initMatrix(A);
		mat->initMatrix(Macc);

		mean_src = 0.0;
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
			
		   		X[0] = (real) w;
				X[1] = (real) h;
				X[2] = 1.0;

				// X_i . X_i^T
				mat->vectorialProduct(Macc, X, X);

				// \sigma X_i . X_i^T
				mat->addMatrix(A, Macc);

				// \sigma I(x_i, y_i) * X_i
				U[0] += src[i] * X[0];
				U[1] += src[i] * X[1];
				U[2] += src[i] * X[2];
			
				mean_src += src[i];
			}
		}
		mean_src /= input_size;

		//
		// Inverse the convariance matrix
		mat->initMatrix(A_1);
		mat->inverse(A_1, A);

		//
		// Computes linear plane coordinates (a, b, c) such that z = ax+by+c
		mat->initVector(W);

		// W = A^-1.U  or  W = U.(A^-1)^T
		mat->transposeMatrix(A_1);
		mat->product(W, U, A_1);
		
		//
		a = W[0];
		b = W[1];
		c = W[2];

		//
		// Create the light plane
	
		if(verbose) 
			print(" + mean src = %g\n", mean_src);
	
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
			
				linearplane[i] = a * (float) w + b * (float) h + c;
		
				real z = src[i] - (linearplane[i] - mean_src);			
				if(z < 0.0) z = 0.0;
				if(z > 255.0) z = 255.0;
				
				dst[i] = z;
			}
		}
		break;
	case 1:
	case 2:
		message("ipNormBestFitLinear::process() not implemented for RGB, YUV and float format");
		break;
	}
}

ipNormBestFitLinear::~ipNormBestFitLinear()
{
	delete mat;
}

}

