#ifndef IPSTAT_INC
#define IPSTAT_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to process a gray image and to compute the mean and the variance of pixels

    \begin{verbatim}

         +---+          +--------------+         
         |XXX|	        |              |         
	 |XXX|   ---->  |    ipStat    | ---->  seq_out->frames[0][0] contains the mean 
         |XXX|          |              |        seq_out->frames[0][1] contains the variance
         +---+          +--------------+        

	 image                                   
	 
    \end{verbatim}

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipStat : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipStat(int width_, int height_, const char *coding_);

	///
	virtual ~ipStat();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
