#ifndef IPSUM_INC
#define IPSUM_INC

#include "ipCore.h"

namespace Torch {

/** process an image and computes the sum of pixels.

    \begin{verbatim}

         +---+          +--------------+         
         |XXX|	        |              |         
	 |XXX|   ---->  |     ipSum    | ---->  seq_out->frames[0][0] contains the sum
         |XXX|          |              |        
         +---+          +--------------+        

	 image                                   
	 
    \end{verbatim}

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipSum : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
public:
	/** constructor
	
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipSum(int width_, int height_, const char *coding_);

	///
	virtual ~ipSum();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
