#include "ipZoom.h"

namespace Torch {

ipZoom::ipZoom(int width_, int height_, const char *coding_, int zoom_factor_) 
	: ipCore(coding_)
{
	width = width_;
	height = height_;
	
   	zoom_factor = zoom_factor_;

	width_out = zoom_factor * width;
	height_out = zoom_factor * height;

	seq_out = new(allocator) Sequence(1, width_out*height_out*n_planes);
}

void ipZoom::process(Sequence *seq_in)
{
	switch(coding_id)
	{
	case 0:
		for(int h = 0 ; h < height_out ; h++)
		{
			for(int w = 0 ; w < width_out ; w++)
			{
				int offset_out = h * width_out + w;
				int offset_in = h/zoom_factor * width + w/zoom_factor;
			   
				seq_out->frames[0][offset_out] = seq_in->frames[0][offset_in];
			}
		} 
		break;
	case 1:
	case 2:
		for(int h = 0 ; h < height_out ; h++)
		{
			for(int w = 0 ; w < width_out ; w++)
			{
				int offset_out = 3 * (h * width_out + w);
				int offset_in = 3 * (h/zoom_factor * width + w/zoom_factor);
			   
				seq_out->frames[0][offset_out] = seq_in->frames[0][offset_in];
				seq_out->frames[0][offset_out+1] = seq_in->frames[0][offset_in+1];
				seq_out->frames[0][offset_out+2] = seq_in->frames[0][offset_in+2];
			}
		} 
		break;
	case 3:
		for(int h = 0 ; h < height_out ; h++)
		{
			for(int w = 0 ; w < width_out ; w++)
			{
				int offset_out = h * width_out + w;
				int offset_in = h/zoom_factor * width + w/zoom_factor;
			   
				seq_out->frames[0][offset_out] = seq_in->frames[0][offset_in];
			}
		} 
		break;
	}
}

int ipZoom::getWidthOut(void)
{
        return width_out;
}

int ipZoom::getHeightOut(void)
{
        return height_out;
}


ipZoom::~ipZoom()
{
}

}

