#include "vision.h"

namespace Torch {

int iIndex(int y, int x, int width)
{
     return y*width+x;
}

double log2(double a)
{
   	return log10(a)/log10(2.0);
}

bool cropGray(real *src_, int width_, int height_, int x_, int y_, int w_, int h_, real *dest_)
{
	if(src_ == NULL) return false;
	if(dest_ == NULL) return false;

	int image_size_ = width_ * height_;

	if(h_ > height_) return false;
	if(w_ > width_) return false;

	int iPixelIn, iPixelIn_, iPixelOut;
	int iLine, iColumn;
	int offset1, offset2, offset3;
	
	offset1 = x_ + y_ * width_;
	
	for(iLine = 0 ; iLine < h_ ; iLine++)
	{
	   	offset2 = iLine * w_;
		offset3 = iLine * width_;
                iPixelIn_ = offset1 + offset3;
		
        	for(iColumn = 0 ; iColumn < w_ ; iColumn++)
		{
                	iPixelOut = iColumn + offset2;
                	iPixelIn = iPixelIn_ + iColumn;

                	if(iPixelIn < image_size_) 
				dest_[iPixelOut] = src_[iPixelIn];
			else 
				dest_[iPixelOut] = 0.0;
      		}
	}

	return true;
}

bool cropRgb(real *src_, int width_, int height_, int x_, int y_, int w_, int h_, real *dest_)
{
	if(src_ == NULL) return false;
	if(dest_ == NULL) return false;

	int image_size_ = 3 * width_ * height_;

	if(h_ > height_) return false;
	if(w_ > width_) return false;

	int iPixelIn, iPixelIn_, iPixelOut;
	int iLine, iColumn;
	int offset1, offset2, offset3;
	
	offset1 = x_ + y_ * width_;
	
	for(iLine = 0 ; iLine < h_ ; iLine++)
	{
	   	offset2 = iLine * w_;
		offset3 = iLine * width_;
                iPixelIn_ = offset1 + offset3;
		
        	for(iColumn = 0 ; iColumn < w_ ; iColumn++)
		{
                	iPixelOut = iColumn + offset2;
                	iPixelIn = iPixelIn_ + iColumn;

			iPixelOut *= 3;
			iPixelIn *= 3;
			   
                	if(iPixelIn < image_size_) 
			{
			   	dest_[iPixelOut] = src_[iPixelIn];
			   	dest_[iPixelOut+1] = src_[iPixelIn+1];
			   	dest_[iPixelOut+2] = src_[iPixelIn+2];
			}
			else 
			{
				dest_[iPixelOut] = 0.0;
				dest_[iPixelOut+1] = 0.0;
				dest_[iPixelOut+2] = 0.0;
			}
      		}
	}

	return true;
}

bool pasteGray(real *src_, int x_, int y_, int w_, int h_, real *dest_, int width_, int height_)
{
	if(src_ == NULL) return false;
	if(dest_ == NULL) return false;

	int image_size_ = width_ * height_;

	if(h_ > height_) return false;
	if(w_ > width_) return false;

	int iPixelIn, iPixelIn_, iPixelOut;
	int iLine, iColumn;
	int offset1, offset2, offset3;
	
	offset1 = x_ + y_ * width_;
	
	for(iLine = 0 ; iLine < h_ ; iLine++)
	{
	   	offset2 = iLine * w_;
		offset3 = iLine * width_;
                iPixelIn_ = offset1 + offset3;
		
        	for(iColumn = 0 ; iColumn < w_ ; iColumn++)
		{
                	iPixelOut = iColumn + offset2;
                	iPixelIn = iPixelIn_ + iColumn;

                	if(iPixelIn < image_size_) 
				dest_[iPixelIn] = src_[iPixelOut];
      		}
	}

	return true;
}

void scaleGray(int w, int h, real *image, real scale_)
{
	real max_ = -INF;
	real min_ = INF;
	real *image_ = image;
	
	for(int i = 0 ; i < h ; i++)
		for(int j = 0 ; j < w ; j++)
		{
			if(*image_ > max_) max_ = *image_;
			if(*image_ < min_) min_ = *image_;

			image_++;
		}
	
	image_ = image;
	
	for(int i = 0 ; i < h ; i++)
		for(int j = 0 ; j < w ; j++)
		{
			real z = (real) scale_ * (*image_ - min_) / (max_ - min_);
			*image_++ = z;
		}
}
	

}

