const char *help = "\
progname: readbindata.cc\n\
code2html: This program reads a bindata file (Torch binary dataset format).\n\
version: Torch3 vision2.0, 2003-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char *argv[])
{
	char *filename;
	bool verbose;
	bool brackets;

	CmdLine cmd;
	cmd.setBOption("write log", false);

	//
	cmd.info(help);
	cmd.addText("\nArguments:");
	cmd.addSCmdArg("filename", &filename, "image file in");
	cmd.addText("\nOptions:");
	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
	cmd.addBCmdOption("-brackets", &brackets, false, "print brackets");
	cmd.read(argc, argv);
	
	//
	int n_inputs;
	int n_patterns;
	DiskXFile *pf_in;

	pf_in = new DiskXFile(filename, "r");
	if((pf_in == NULL) || (pf_in->is_opened == false))
	{
      		error("Opening bindata file %s", filename);
      
      		return 0;
	}
	
	pf_in->read(&n_patterns, sizeof(int), 1);
	pf_in->read(&n_inputs, sizeof(int), 1);

	if(!verbose)
	{
		print("Reading bindata file (%s)\n", filename);
		print("   n_inputs = %d\n", n_inputs);
		print("   n_patterns = %d\n", n_patterns);  
	}

	real data;

	if(verbose)
	{
		for(int j = 0 ; j <  n_patterns ; j++)
		{
			if(brackets) print("[%06d] = [ ", j);
		
			for(int i = 0 ; i <  n_inputs ; i++)
			{
				pf_in->read(&data, sizeof(real), 1);   
				print("%g ", data);
			}
	
			if(brackets) print("]\n");
			else print("\n");
		}	
		delete pf_in;
	}

	return 0;
}
