#ifndef PATTERN_INC
#define PATTERN_INC

#include "Object.h"

namespace Torch {

/** This class is designed to handle patterns detected in images
  
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class Pattern : public Object
{
public:
	/// scale of the pattern
	real scale;

	/// view of the pattern
	int view;
	
	/// confidence (could be interpreted as a probability)
	real confidence;

	/// another confidence related to overlaps
	real activation;

	/// x location of the pattern
	int x;
	
	/// y location of the pattern
	int y;
	
	/// width of the pattern
	int w;

	/// height of the pattern
	int h;

	//---
	
	///
	Pattern(int width_=0, int height_=0);

	///
	virtual void reset();

	///
	virtual void sprint();
	
	///
	virtual void copyFrom(Pattern *);

	///
	~Pattern() {};
};

/** Functions to compare patterns

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
//@{

/// comparing function to sort patterns according to their size from smaller to bigger
int sortPatternSurfInc(const void *e1, const void *e2);

/// comparing function to sort patterns according to their size from bigger to smaller
int sortPatternSurfDec(const void *e1, const void *e2);

/// comparing function to sort patterns according to their confidence from larger to smaller
int sortPatternConfDec(const void *e1, const void *e2);

/// comparing function to sort patterns according to their activation from larger to smaller
int sortPatternActivDec(const void *e1, const void *e2);
	
/** compute the overlapping surface between two patterns

    @return the overlapping surface
*/
real SurfOverlap(Pattern *p1, Pattern *p2);
				
/** test if a pattern is included into another one
 
    @return true if p1 is included in p2
*/
bool rectIsInRect(Pattern *p1, Pattern *p2);

//@}

}

#endif

