#ifndef REAL_STUMP_MACHINE_INC
#define REAL_STUMP_MACHINE_INC

#include "Machine.h"

namespace Torch {

/** This class is designed to train/support real weak classifiers

    A real weak classifier has as parameters:
    \begin{itemize}
    \item the index of the feature to test #feature_id#,
    \item the range of the feature outputs #min#, #max#,
    \item a look up table with #n_bins#.
    \end{itemize}

    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class RealStumpMachine : public Machine
{   
protected:
	// explicit parameters of the machine 
	int feature_id;
	real min, max;
	real *lut;

public:
        /// number of bins of the lut
	int n_bins;
	
       	/// number of possible features
        int n_features;

	//-----
	
	///
	RealStumpMachine(int n_features_ = 0, int n_bins_ = 100);	

	/// decode the parameters of the machine
	virtual void setParams(int feature_id_, real min_, real max_, real *lut_);

	/// compute the feature #feature_id_# for the sequence #inputs_# 
        virtual real getFeature(int feature_id_, Sequence *inputs_);

	/// computes the output of the machine given an input sequence
	virtual void forward(Sequence *inputs);

	/// load the parameters of the machine
	virtual void loadXFile(XFile *file);

	/// save the parameters of the machine
	virtual void saveXFile(XFile *file);

	///	
	virtual ~RealStumpMachine();
};

}

#endif
