#ifndef SONY_EVI_D100P_INC
#define SONY_EVI_D100P_INC

#include "ViscaSerial.h"

namespace Torch {

/** This class is designed to control a SONY EVI D100P using VISCA serial interface

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class SonyEviD100P : public ViscaSerial
{
   	int pan_min;
        int pan_max;
	int pan_center;
        int tilt_min;
        int tilt_max;
	int tilt_center;

public:
   	/// pan speed (from 1 to 18)
   	int pan_speed;
	
   	/// tilt speed (from 1 to 14)
	int tilt_speed;
	
	//-----
	
	///
   	SonyEviD100P(char *device_, char port_);

	///
	virtual ~SonyEviD100P();

	///
	int getPanSpeedMin() { return 1; };

	///
	int getPanSpeedMax() { return 18; };
	
	///
	void setPanSpeed(int pan_speed_) { pan_speed = pan_speed_; };

	///
	int getTiltSpeedMin() { return 1; };

	///
	int getTiltSpeedMax() { return 14; };

	///
	void setTiltSpeed(int tilt_speed_) { tilt_speed = tilt_speed_; };

	/// get the power status of the camera
	int getPowerStatus();

	/// set power ON
	bool setPowerOn();

	/// set power OFF
	bool setPowerOff();

	/// BackLight compensation ON
	bool setBackLightOn();

	/// BackLight compensation OFF
	bool setBackLightOff();
	
	/** @name Pan/Tilt controls */
	//@{
	bool goHome();

	///
	bool up();

	///
	bool down();

	///
	bool left();

	///
	bool right();

	///
	bool upLeft();

	///
	bool upRight();

	///
	bool downLeft();

	///
	bool downRight();

	///
	bool stopPanTilt();

	///
	bool goRelative(int x, int y, int width, int height);

	///
	bool goAbsolute(int x, int y, int width, int height);

	///
	bool goRelative(int pan, int tilt);

	///
	bool goAbsolute(int pan, int tilt);
	//@}
	
	/** @name Zoom controls */
	//@{
	///
	bool zoomTeleStd();
	
	///
	bool zoomWideStd();
	
	///
	bool stopZoom();
	//@}
};

}

#endif
