#include "jesorskyGTMeasurer.h"

namespace Torch {

jesorskyGTMeasurer::jesorskyGTMeasurer(FaceModel *facemodel_, bool oneface_, real threshold_) : FaceDetectGTMeasurer(facemodel_, oneface_)
{
	threshold = threshold_;
}
	
real jesorskyGTMeasurer::jesorsky(sPoint2D *l_eye, sPoint2D *r_eye, sPoint2D *l_eye_gt, sPoint2D *r_eye_gt)
{
   	real z1, z2, d1, d2, d, EE;

	// squared distance between left eye and gt left eye
	z1 = (l_eye->x - l_eye_gt->x);
	z2 = (l_eye->y - l_eye_gt->y);
	d1 = z1*z1 + z2*z2;

	// squared distance between right eye and gt right eye
	z1 = (r_eye->x - r_eye_gt->x);
	z2 = (r_eye->y - r_eye_gt->y);
	d2 = z1*z1 + z2*z2;

	// max of d1 and d2
	d = (d1 > d2 ? d1 : d2);
	
	// squared distance between the gt eyes
	z1 = (r_eye_gt->x - l_eye_gt->x);
	z2 = (r_eye_gt->y - l_eye_gt->y);
	EE = z1*z1+z2*z2;
	
	return sqrt(d / EE);
}

bool jesorskyGTMeasurer::measure(int n_, sPoint2D *shape_, real *the_measure)
{
	if(n_ != 2 && n_ != 4) 
		error("jesorskyGTMeasurer::measure() the shape should be made of 2 or 4 points");

	sPoint2D l_eye;
	sPoint2D r_eye;

	if(n_ == 2)
	{
	   	if(verbose) print("Measure from 2 points (eye centers)\n");
		
		l_eye = shape_[0];
		r_eye = shape_[1];
	}
	else if(n_ == 4)
	{
	   	if(verbose) print("Measure from 4 points (bbx)\n");
		
		sPoint2D eyes[2];
		
		facemodel->bbx2ldm(n_, shape_, 2, eyes);
		
		l_eye = eyes[0];
		r_eye = eyes[1];
	}

	for(int i = 0 ; i < n_faces ; i++)
	{
		real J = jesorsky(&l_eye, &r_eye, &l_eyes[i], &r_eyes[i]);
		if(verbose)
	   		print("J({l_eye = (%g, %g), r_eye = (%g, %g)} == {l_eyes[%d] = (%g, %g), r_eyes[%d] = (%g, %g)}) = %g\n", l_eye.x, l_eye.y, r_eye.x, r_eye.y, i, l_eyes[i].x, l_eyes[i].y, i, r_eyes[i].x, r_eyes[i].y, J);

		if(J <= threshold)
		{
			if(verbose)
				print("jesorskyGTMeasurer::measure() Match found (error=%g)\n", J);

			if(the_measure != NULL) *the_measure = J;

			return true;
		}
	}

	if(verbose)
		print("jesorskyGTMeasurer::measure() No match found\n");

	if(the_measure != NULL) *the_measure = -1.0;
	
	return false;
}

jesorskyGTMeasurer::~jesorskyGTMeasurer()
{
}

}
